package jesse.keeblarcraft.CustomBlocks.BlockEntities;

import jesse.keeblarcraft.GuiMgr.FactionBlockScreenHandler;
import jesse.keeblarcraft.world.ImplementedInventory;
import net.fabricmc.fabric.api.screenhandler.v1.ExtendedScreenHandlerFactory;
import net.minecraft.block.BlockState;
import net.minecraft.block.entity.BlockEntity;
import net.minecraft.entity.player.PlayerEntity;
import net.minecraft.entity.player.PlayerInventory;
import net.minecraft.inventory.Inventories;
import net.minecraft.item.ItemStack;
import net.minecraft.nbt.NbtCompound;
import net.minecraft.network.PacketByteBuf;
import net.minecraft.screen.PropertyDelegate;
import net.minecraft.screen.ScreenHandler;
import net.minecraft.server.network.ServerPlayerEntity;
import net.minecraft.text.Text;
import net.minecraft.util.collection.DefaultedList;
import net.minecraft.util.math.BlockPos;
import net.minecraft.world.World;

public class FactionBlockEntity extends BlockEntity implements ExtendedScreenHandlerFactory, ImplementedInventory {
    private final DefaultedList<ItemStack> inventory = DefaultedList.ofSize(7, ItemStack.EMPTY);

    private static final int DEFENSE_SLOT_ONE = 0;
    private static final int DEFENSE_SLOT_TWO = 1;
    private static final int OFFENSE_SLOT_ONE = 2;
    private static final int OFFENSE_SLOT_TWO = 3;
    private static int factionPower = 0;

    protected final PropertyDelegate propertyDelegate;

    public FactionBlockEntity(BlockPos pos, BlockState state) {
        super(BlockEntityRegistration.FACTION_BLOCK_ENTITY, pos, state);
        this.propertyDelegate = new PropertyDelegate() {

            @Override
            public int get(int index) {
                // The only value we need to get/delegate is faction power
                return switch(index) {
                    default -> factionPower;
                };
            }

            @Override
            public void set(int index, int value) {
                switch(index) {
                    default -> factionPower = value;
                };
            }

            @Override
            public int size() {
                // We are only sync'ing 1 integer - faction power
                return 1;
            }

        };
    }

    @Override
    public Text getDisplayName() {
        return Text.literal("Faction Home Base Station"); // Replace with proper en_us format later
    }

    @Override
    public ScreenHandler createMenu(int syncId, PlayerInventory playerInventory, PlayerEntity player) {
        return new FactionBlockScreenHandler(syncId, playerInventory, this, this.propertyDelegate);
    }

    @Override
    public void readNbt(NbtCompound nbt) {
        super.readNbt(nbt);
        Inventories.readNbt(nbt, inventory);
        factionPower = nbt.getInt("faction_block_entity.power");
    }

    @Override
    public void writeNbt(NbtCompound nbt) {
        super.writeNbt(nbt);
        // Write the inventory of the block
        Inventories.writeNbt(nbt, inventory);
        nbt.putInt("faction_block_entity.power", factionPower);
    }

    @Override
    public DefaultedList<ItemStack> getItems() {
        return inventory;
    }

    @Override
    public void writeScreenOpeningData(ServerPlayerEntity player, PacketByteBuf buf) {
        // Send position of this block entity from server->client->server etc
        buf.writeBlockPos(this.pos);
    }

    // Tick method is called 20 times a second
    public void tick(World world, BlockPos pos, BlockState state) {
        // For reasons unknown to me and only to Kaupenjoe (youtube video) - we never want to call this on a client.
        if (world.isClient()) {
            return;
        }

        // Do stuff here that we need to do on a per tick basis. Probably check the items inside the slots and do stuff?
        // Maybe something with callback handlers might be done here? Like the temp flight within bounds of the faction block if that powerup is
        // active, etc?
    }
}
